
/* Burgers' (scalar nonlinear conservation law)
 * Structures and operator overloading for ConservedVars, PrimitiveVars and XtraVars
 */

#ifndef STRUCTURES_BURGERS_H
#define STRUCTURES_BURGERS_H

#include "numtypes.h"

// === ConservedVars

struct ConservedVars {
  real   u;
  
  real norm() const
  {
    return fabs(u);
  }
  
  // defined in "structures.cpp"
  
  void operator += (const ConservedVars &a);
  void operator -= (const ConservedVars &a);
  void operator *= (const ConservedVars &a);
  void operator /= (const ConservedVars &a);
  void operator /= (const real r);
  void operator *= (const real r);
  void operator =  (const real r);
  #if COMPUTE_KURTOSIS
  void operator -= (const real r);
  #endif
};

// === XtraVars

struct XtraVars {
  real   E;
  
  // defined in "structures.cpp"
  
  void operator += (const XtraVars &a);
  void operator -= (const XtraVars &a);
  void operator *= (const XtraVars &a);
  void operator /= (const XtraVars &a);
  void operator /= (const real r);
  void operator *= (const real r);
  void operator =  (const real r);
  #if COMPUTE_KURTOSIS
  void operator -= (const real r);
  #endif
};

// === PrimitiveVars

struct PrimitiveVars {
  real   u;
  
  void toConservedVars(ConservedVars &v) const {
    v.u = u;
  }
  
  PrimitiveVars() {}
  PrimitiveVars(const ConservedVars &CV, const XtraVars &XV);
};

// picks PrimitiveVars from ConservedVars (U) and XtraVars (Xtra)
inline PrimitiveVars::PrimitiveVars(const ConservedVars &CV, const XtraVars &XV)
{
  u = CV.u;
}

#endif
