
#include "topography.h"
#include "ec_util.h"

#include "equation.h"
#include "multi.h"

real b_cell_center (int i, int j) {
  // cell centers
  real x = (i + 0.5 + NXl*multix) * dx[X];
  real y;
  if (HAVE_DIR(Y))
    y = (j + 0.5 + NYl*multiy) * dx[Y];
  else
    y = 1.0;
  return b(x, y);
}

#if EC_TOPOGRAPHY_QUAD

// 2nd order accurate energy conservative (EC) source quadrature
void compute_source (int i, int j) {
  Source[i][j][0].hu[X] -= avg ( U[i]  [j][0].h, U[i+1][j][0].h ) * jump( b_cell_center(i,  j), b_cell_center(i+1,j) );
  Source[i][j][0].hu[X] -= avg ( U[i-1][j][0].h, U[i]  [j][0].h ) * jump( b_cell_center(i-1,j), b_cell_center(i,  j) );
  Source[i][j][0].hu[X] *= GRAVITY / (2*dx[X]);
  if (HAVE_DIR(Y))
    Source[i][j][0].hu[Y] -= avg ( U[i][j]  [0].h, U[i][j+1][0].h ) * jump( b_cell_center(i,j  ), b_cell_center(i,j+1) );
    Source[i][j][0].hu[Y] -= avg ( U[i][j-1][0].h, U[i][j]  [0].h ) * jump( b_cell_center(i,j-1), b_cell_center(i,j  ) );
    Source[i][j][0].hu[Y] *= GRAVITY / (2*dx[Y]);
}

#else

// 2nd order accurate source quadrature
void compute_source (int i, int j) {
  Source[i][j][0].hu[X] = - GRAVITY * U[i][j][0].h * ( b_cell_center(i+1,j) - b_cell_center(i-1,j) ) / (2*dx[X]);
  if (HAVE_DIR(Y))
    Source[i][j][0].hu[Y] = - GRAVITY * U[i][j][0].h * ( b_cell_center(i,j+1) - b_cell_center(i,j-1) ) / (2*dx[Y]);
}

#endif
