
#include "topography.h"

#include "equation.h"
#include "multi.h"
#include "chaos.h"

///::title:: Karhunen-Loeve expansion (2D version)
///::vars:: BED_OFFSET=0.0 KL=16
///::consts:: T_UNIFORM=KL T_NORMAL=0

#define KLX KL/2
#define KLY KL - KLX

real b (real x, real y) {
  // Karhunen–Loeve expansion for topography b consisting of KL basis functions
  real value = BED_OFFSET;
  for (int mx=1; mx<=KLX; mx++)
    for (int my=1; my<=KLY; my++)
      value += 0.25 * pow(mx,-1.1)*sin(mx*pi*x)*(2*chaos.t_uniform[mx-1] - 1) * pow(my,-1.1)*sin(my*pi*y)*(2*chaos.t_uniform[KLX+my-1] - 1);
  return value;
}
